#!/usr/bin/perl
# 
# JamNNTPd log analyzer version 2.0 by Johan Billing
# This program has been released into the public domain, use it any way you want.
#
# Requires the TimeDate module.

use Time::Local;
use Date::Parse;
use Date::Format;
use Getopt::Long;
use Text::ParseWords;

$getopt_res = GetOptions(\%opts, 'days|d=i', 
                                 'maxposters|mp=i', 
                                 'maxusers|mu=i',
                                 'maxip|mi=i',
                                 'users|u=s',
                                 'displayip|di', 
                                 'noheader|nh', 
                                 'nodates|nd', 
                                 'nototals|nt', 
                                 'noirreg|ni', 
                                 'nofull|nf', 
                                 'nodenied|nd', 
                                 'help|h');
                                 
if ( !$getopt_res or $opts{help} or $#ARGV != 0) {
   print "\n";
   print "Usage: [perl] loganalyzer.pl [<options>] <logfile>\n";
   print "\n";
   print "Valid options are:\n";
   print "\n";
   print " --d=X  or --days=X        Only analyze today and last X days in logfile\n";
   print " --mp=X or --maxposters=X  Maximum number of entries in poster toplist\n";
   print " --mu=X or --maxusers=X    Maximum number of entries in users toplist\n";
   print " --mi=X or --maxip=X       Maximum number of entries in IP toplist\n";
   print " --u=X  or --users=X       Resolve usernames using jamnntpd.users file X\n";
   print " --di   or --displayip     Display IP addresses instead of resolved names\n";
   print " --nh   or --noheader      Do not include header in output\n";
   print " --nd   or --nodates       Do not include dates in output\n";
   print " --nt   or --nototals      Do not include totals in output\n";
   print " --ni   or --noirreg       Do not include irregular terminations in output\n";
   print " --nf   or --nofull        Do not include times server full in output\n";
   print " --nd   or --nodenied      Do not include times access was denied in output\n";
   print " --h    or --help          Show this information\n";
   print "\n";
   
   exit (0);
}                   
                   
if ( defined $opts{days} and $opts{days} ) {
   ($sec,$min,$hour,$mday,$mon,$year) = localtime();
   $opts_starttime = timelocal(0,0,0,$mday,$mon,$year) - $opts{days}*24*60*60;
}

if ( $opts{users} ) {
   open (USERSFILE, $opts{users}) or die "Cannot open $opts{users}";

   while (<USERSFILE>) {
      s/\s+$//; # strip trailing whitespace
      
      if ( ! /^#/ ) {
         @words = quotewords('\s+', 0, $_);
         
         if ( $words[0] and $words[4] ) {
            @names = split(/,/,$words[4]);
      
            if ( ! ( $names[0] =~ /(\?|\*)/ ) ) {
               $realnames{$words[0]} = $names[0]}
         }
      }
   }
   close (USERSFILE);
}


$logfile = $ARGV[0];

# Read logfile

$irregular = 0;
$serverfull = 0;
$denied = 0;

$totaltime = 0;
$totalconn = 0;
$totalpost = 0;

open (LOGFILE, $logfile) or die "Cannot open $logfile";

while (<LOGFILE>) {
   s/\s+$//; # strip trailing whitespace

   $message = "";

   if (/^(\S+ \S+) \((\S+)\) (.*)/) {
      ($datetime, $session, $message) = ($1,$2,$3);
   }
   elsif (/^(\S+ \S+) (.*)/) {
      $session = "";
      ($datetime, $message) = ($1,$2);
   }

   if( !$opts_starttime or str2time($datetime) > $opts_starttime ) {
   
      $lasttime = $datetime;

      if ( !$firsttime) {
         $firsttime = $datetime;
      }
  
      if ( $message =~ /is running on port/ ) {
         if ( $running ) {
            $irregular += 1;
         }
      
         $running = 1;
      
         %starttimes = ();
         %loginnames = ();      
      }

      elsif ( $message =~ / exited/ ) {
         $running = 0;
      }
   
      elsif ( $message =~ /Connection established to/ ) {
         $starttimes{$session} = str2time($datetime);
         $loginnames{$session} = "";
      
         @dummy = split(/:/,$session);
         $iplookup{$dummy[0]} = substr($message,26);
      }

      elsif ( $message =~ /server full/ ) {
         $serverfull += 1;
      }

      elsif ( $message =~ /not in allow list/ ) {
         $denied += 1;
      }
         
      elsif ( $message =~ /Logged in as/ ) {
         $loginnames{$session} = substr($message,13);
      }

      elsif ( $message =~ /Posted message to/ ) {
         $name = $loginnames{$session};
      
         $namepost{$name} += 1;
         $totalpost += 1;
      }
   
      elsif ( $message =~ /Connection closed/ ) {
         if ( $starttimes{$session} ) {
            $name = $loginnames{$session};
            $time = str2time($datetime) - $starttimes{$session};

            if ($time < 0) {
               print STDERR "WARNING: Session $session from ", time2str('%d-%b-%y %H:%M:%S',$starttimes{$session}), " to $datetime has negative duration\n";
            }
            elsif ($time > 24*60*60) {
               print STDERR "WARNING: Session $session from ", time2str('%d-%b-%y %H:%M:%S',$starttimes{$session}), " to $datetime is abnormally long ($time seconds)\n";
            }
            
            $nametime{$name} += $time;
            $nameconn{$name} += 1;
            
            @dummy = split(/:/,$session);
            $iptime{$dummy[0]} += $time;
            $ipconn{$dummy[0]} += 1;

            $totaltime += $time;
            $totalconn += 1;
      
            delete($loginnames{$session});
            delete($starttimes{$session});
         }
      }
   }
}

close (LOGFILE);

# Print results

unless ( $opts{noheader} ) {
   print "\n";
   print "Summary of JamNNTPd logfile $logfile\n";
}

unless ( $opts{nodates} ) {
   print "\n";
   print "First entry: $firsttime\n";
   print " Last entry: $lasttime\n";
}

unless ( defined $opts{maxusers} and $opts{maxusers} == 0 ) {
   print "\n";
   print "Top users:\n";
   print "\n";

   $num=1;

   foreach $key (sort { $nametime{$b} <=> $nametime{$a} } (keys (%nametime))) {
      unless (defined $opts{maxusers} and $num > $opts{maxusers} ) {
         $name = $key;
 
         if ( !$name ) {
            $name = "<Not logged in>";
         }
         elsif ( $realnames{$name} ) {
            $name = $realnames{$name};
         }
         
         $secs = $nametime{$key};
         
         printf ( "%3d: %-40.40s %5d:%.02d:%.02d in %5d connections\n", $num++, $name, $secs / (60 * 60), $secs / 60 % 60, $secs % 60, $nameconn{$key} );
      }
   }
}

unless ( defined $opts{maxip} and $opts{maxip} == 0 ) {
   print "\n";
   print "Top IP addresses:\n";
   print "\n";

   $num=1;

   foreach $key (sort { $iptime{$b} <=> $iptime{$a} } (keys (%iptime))) {
      unless (defined $opts{maxip} and $num > $opts{maxip} ) {
         $addr = $key;
         
         unless ( $opts{displayip} ) {
            $addr = $iplookup{$addr};
         }
         
         $secs = $iptime{$key};
        
         printf ( "%3d: %-40.40s %5d:%.02d:%.02d in %5d connections\n", $num++, $addr, $secs / (60 * 60), $secs / 60 % 60, $secs % 60, $ipconn{$key} );
      }
   } 
}

unless ( defined $opts{maxposters} and $opts{maxposters} == 0 ) {
   print "\n";
   print "Top posters:\n";
   print "\n";

   $num=1;

   foreach $key (sort { $namepost{$b} <=> $namepost{$a} } (keys (%namepost))) {
      unless (defined $opts{maxposters} and $num > $opts{maxposters} ) {
         $name = $key;
 
         if ( !$name ) {
            $name = "<Not logged in>";
         }
         
         elsif ( $realnames{$name} ) {
            $name = $realnames{$name};
         }
 
         printf ( "%3d: %-40.40s %5d messages\n", $num++, $name, $namepost{$key} );
      }
   }
}

unless ( $opts{nototals} ) {
   print "\n";
   printf "Total connection time: %d:%.02d:%.02d in %d connections\n" => $totaltime / (60 * 60), $totaltime / 60 % 60, $totaltime % 60,$totalconn;
   print "Total posted messages: $totalpost\n";
}

unless ( $opts{nofull} ) {
   print "\n";
   print "Server was full $serverfull times.\n";
}

unless ( $opts{nodenied} ) {
   print "\n";
   print "Access was denied $denied times.\n";
}

unless ( $opts{noirreg} ) {
   print "\n";
   print "JamNNTPd terminated irregularly $irregular times.\n";
}

print "\n";
